<?php
/**
 * 2018-2022 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2018-2022 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */


/**
 * NOTICE:
 * This module was inspired to the Prestashop module "ps_featuredproducts"
 * "ps_featuredproducts" was used as a basic code.
 * Starting from its original structure, mxb_featuredproducts has many different features and supports css, html and tags to insert products sliders into CMS pages and into "mxb_customblocks" blocks
 *
 * Also its structure has been upgraded to easy duplicate and customize the module. 
 * Installing 2 or more customized copies of this module, you can enable/disable it for different customers or put it into different sections of the website
 */

/**
* ###### DUPLICATE & CUSTOMIZE 1/3: 
*        you are customizing "mxb_featuredproducts_one"
*        where "one" is your [customname]
*        if you want to create "mxb_featuredproducts_bestseller", "bestseller" will be your [customname]
*        [customname] in file and folder name MUST be a string without underscores and in lowercase. YES: "bestseller" - NO: "BestSeller", "best_seller", "best-seller", "bestseller22"
*        [Customname] in Class name MUST be capitalized. YES: "Customname", "Bestseller" ....
*        [special extensions tpl_, js_, css_, translations/*.php_]: that files are ready to be installed with the customized name of the module. You can change it as you wish but remember to leave the "_mxb_[module]_" key untouched: it will be substituted with the right module name during the installation.
*
*        remember to change:
*            the module folder name to "mxb_featuredproducts_[customname]"
*              file "mxb_featuredproducts_[customname].php":
*                the class name "class Mxb_FeaturedProducts_[Customname]"
*                the CUSTOM_NAME const value in class declaration
*            file "config.xml":
*                the module name value to "mxb_featuredproducts_[customname]"
*            file "controller/front/ajax.php":
*                the controller class name to "Mxb_FeaturedProducts_[Customname]AjaxFrontController"
*/

/**
 * for DEBUG: if you use mxb_tools module, you'll find the log file in folder: /module/mxb_tools/log/
 *         require_once _PS_MODULE_DIR_.'mxb_tools/classes/MxbToolsDebug.php';
 * and then use this function:
 *        MxbToolsDebug::WriteLog($classname, $function, $message, $bypass_settings = false)
 *    i.e: 
 *        MxbToolsDebug::WriteLog($this->name, 'className::__construct()', 'START', true);
 */
 
 
if (!defined('_PS_VERSION_')) {
    exit;
}

//    require_once _PS_MODULE_DIR_.'mxb_tools/classes/MxbToolsDebug.php';
use PrestaShop\PrestaShop\Core\Module\WidgetInterface;
use PrestaShop\PrestaShop\Adapter\Category\CategoryProductSearchProvider;
use PrestaShop\PrestaShop\Adapter\Search\SearchProductSearchProvider;
use PrestaShop\PrestaShop\Adapter\BestSales\BestSalesProductSearchProvider;
use PrestaShop\PrestaShop\Adapter\Manufacturer\ManufacturerProductSearchProvider;

use PrestaShop\PrestaShop\Adapter\Image\ImageRetriever;
use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
use PrestaShop\PrestaShop\Core\Product\ProductListingPresenter;
use PrestaShop\PrestaShop\Adapter\Product\ProductColorsRetriever;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchContext;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchQuery;
use PrestaShop\PrestaShop\Core\Product\Search\SortOrder;


 /**
  * ###### DUPLICATE & CUSTOMIZE 2/3: class declaration: change the class name as the module name: Mxb_Featuredproducts_[Customname] 
  */
class Mxb_FeaturedProducts_One extends Module implements WidgetInterface
{
    /**
    * ###### DUPLICATE 3/3:    set CUSTOM_NAME as "yourname" in lowercase
    *                        set $mainHook as you wish: it is the default hook for this module
    */
    const CUSTOM_NAME     = 'one';
    public $mainHook     = 'displayHome';    
    /**
       * ###### DUPLICATE & CUSTOMIZE: no more changes are needed from here
     */
    
    const MODULE_NAME_ROOT = 'mxb_featuredproducts';
    public $documentationUrl = 'https://www.mxbinformatica.it/prestashop-mxb-featuredproducts/';
    public $thumbnailUrl = 'https://www.mxbinformatica.it/ps_modules/thumbs/mxb_featuredproducts.jpg';

    private $templateFile;

    public function __construct()
    {
        $this->customName         = self::CUSTOM_NAME;
        $this->rootName         = self::MODULE_NAME_ROOT . '_';
        $this->name             = $this->rootName . $this->customName;
        $this->displayName         = 'MXB featured products: ' . $this->customName;
        $this->description         = $this->l('Displays featured products in the central column of your homepage.');
        $this->version             = '1.0.2';
        $this->author             = 'PrestaShop - customized by MXB Informatica';
        $this->need_instance    = 0;
        $this->bootstrap         = true;
        $this->ajax             = true;
        $this->ps_versions_compliancy = ['min' => '1.7.4.0', 'max' => _PS_VERSION_];
        
        $this->customCssFolder             = "customcss";
        $this->templateFile             = 'module:' . $this->name . '/views/templates/hook/mxb_featuredproducts.tpl';
        $this->product_miniature_tpl     = _PS_THEME_DIR_ . 'templates/catalog/_partials/miniatures/product.tpl';
        
        // configuration values names
        $this->HOME_FEATURED_PRODUCT_CATEGORY    = $this->name . '_PRODUCT_CAT';
        $this->HOME_FEATURED_NBR                 = $this->name . '_NBR';
        $this->HOME_FEATURED_CAT                 = $this->name . '_CAT';
        $this->HOME_FEATURED_RANDOMIZE             = $this->name . '_RANDOMIZE';
        $this->HOME_FEATURED_DESC                 = $this->name . '_DESC';
        $this->HOME_FEATURED_ENABLE_TAG            = $this->name . '_ENABLE_TAG';
        $this->HOME_FEATURED_SLIDE                 = $this->name . '_SLIDE';
        $this->HOME_FEATURED_FOOTER_LINK        = $this->name . '_FOOTER_LINK';
        $this->HOME_FEATURED_SEARCH_STRING         = $this->name . '_SEARCH_STRING';
        $this->HOME_FEATURED_BEST_SALES            = $this->name . '_BEST_SALES';
        $this->HOME_FEATURED_CLASS                 = $this->name . '_CLASS';
        $this->HOME_FEATURED_BRAND                 = $this->name . '_BRAND';

        // 05-2023: add free argouments (paragraphs) into tags
        // $this->HOME_FEATURED_TYPE                 = $this->name . '_TYPE';

        // product miniature CSS management
        $this->PRODUCT_CUSTOM_CSS_CONTENT        = $this->name . '_PROD_CUSCSS';
        $this->PRODUCT_CUSTOM_CSS_ENABLE        = $this->name . '_PROD_CUSCSS_FLAG';

        parent::__construct();


        if (Tools::getValue('download_product_tpl')) {
            if (file_exists($this->product_miniature_tpl)) {
                $myfile = fopen($this->product_miniature_tpl, "r");
                $content = fread($myfile, filesize($this->product_miniature_tpl));
                Die('<textarea style="width:100%;height:100%;">' . $content . '</textarea>');
            }
        }


        //$this->installModuleFiles(); // TODO: delete this in production version (it's already called in install() method !!!)
        
    }

    public function install()
    {
        $this->_clearCache('*');
/*
        Configuration::updateValue($this->HOME_FEATURED_NBR, 8);
        Configuration::updateValue($this->HOME_FEATURED_CAT, (int) Context::getContext()->shop->getCategory());
        Configuration::updateValue($this->HOME_FEATURED_RANDOMIZE, false);
*/
        return parent::install()
            && $this->registerHook('header')
            && $this->registerHook('addproduct')
            && $this->registerHook('updateproduct')
            && $this->registerHook('deleteproduct')
            && $this->registerHook('categoryUpdate')
            && $this->registerHook($this->mainHook) //$this->registerHook('displayHome')
            && $this->registerHook('displayOrderConfirmation2')
            && $this->registerHook('displayCrossSellingShoppingCart')
            && $this->registerHook('actionAdminGroupsControllerSaveAfter')
            && $this->installFixtures()
        ;
    }
    
    
    protected function installFixtures()
    {
        $languages = Language::getLanguages(false);

        foreach ($languages as $lang) {
            $this->installFixture((int)$lang['id_lang']);
        }
        $this->installModuleFiles();
        return true;
    }

    protected function installFixture($id_lang, $image = null)
    {
        //$values[$this->banner_img][(int)$id_lang] = $image;
        $values[$this->HOME_FEATURED_NBR] = 4;
        $values[$this->HOME_FEATURED_CAT] = (int) Context::getContext()->shop->getCategory();
        $values[$this->HOME_FEATURED_BRAND] = '';
        $values[$this->HOME_FEATURED_RANDOMIZE] = false;
        $values[$this->HOME_FEATURED_SLIDE] = false;
        $values[$this->HOME_FEATURED_BEST_SALES] = false;
        
        $values[$this->HOME_FEATURED_ENABLE_TAG] = false;
        $values[$this->HOME_FEATURED_PRODUCT_CATEGORY] = false;
        $values[$this->HOME_FEATURED_DESC][(int)$id_lang] = '';
        $values[$this->HOME_FEATURED_SEARCH_STRING][(int)$id_lang] = '';
        $values[$this->HOME_FEATURED_CLASS] = '';
        
        
            
        $values[$this->HOME_FEATURED_FOOTER_LINK] = true;
        $values[$this->PRODUCT_CUSTOM_CSS_ENABLE] = false;
        
        
        
        

        //Configuration::updateValue($this->banner_img,                     $values[$this->banner_img]);
        Configuration::updateValue($this->HOME_FEATURED_NBR,                 $values[$this->HOME_FEATURED_NBR]);
        Configuration::updateValue($this->HOME_FEATURED_CAT,                 $values[$this->HOME_FEATURED_CAT]);
        Configuration::updateValue($this->HOME_FEATURED_BRAND,                 $values[$this->HOME_FEATURED_BRAND]);
        
        Configuration::updateValue($this->HOME_FEATURED_RANDOMIZE,             $values[$this->HOME_FEATURED_RANDOMIZE]);
        Configuration::updateValue($this->HOME_FEATURED_SLIDE,                 $values[$this->HOME_FEATURED_SLIDE]);
        Configuration::updateValue($this->HOME_FEATURED_BEST_SALES,         $values[$this->HOME_FEATURED_BEST_SALES]);
        
        Configuration::updateValue($this->HOME_FEATURED_ENABLE_TAG,            $values[$this->HOME_FEATURED_ENABLE_TAG]);
        Configuration::updateValue($this->HOME_FEATURED_PRODUCT_CATEGORY,    $values[$this->HOME_FEATURED_PRODUCT_CATEGORY]);
        Configuration::updateValue($this->HOME_FEATURED_DESC,                 $values[$this->HOME_FEATURED_DESC]);
        Configuration::updateValue($this->HOME_FEATURED_SEARCH_STRING,         $values[$this->HOME_FEATURED_SEARCH_STRING]);
        Configuration::updateValue($this->HOME_FEATURED_CLASS,                 $values[$this->HOME_FEATURED_CLASS]);
        
            
        Configuration::updateValue($this->HOME_FEATURED_FOOTER_LINK,        $values[$this->HOME_FEATURED_FOOTER_LINK]);
        Configuration::updateValue($this->PRODUCT_CUSTOM_CSS_ENABLE,        $values[$this->PRODUCT_CUSTOM_CSS_ENABLE]);
        
    }
    
    

    public function uninstall()
    {
        $this->_clearCache('*');

        return parent::uninstall();
    }
    
    
    
    
    
    
    
    
     /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path.DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR.'js'.DIRECTORY_SEPARATOR.'front.js');
        $this->context->controller->addCSS($this->_path.DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR.'css'.DIRECTORY_SEPARATOR.'front.css');
        
        if ((bool) Configuration::get($this->PRODUCT_CUSTOM_CSS_ENABLE)) {
            $this->context->controller->addCSS($this->_path.DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR.$this->customCssFolder.DIRECTORY_SEPARATOR.'product.css');
            //$this->context->controller->addCSS($this->_path.'/views/product.css');
        }
    }
    
    

    public function hookAddProduct($params)
    {
        $this->_clearCache('*');
    }

    public function hookUpdateProduct($params)
    {
        $this->_clearCache('*');
    }

    public function hookDeleteProduct($params)
    {
        $this->_clearCache('*');
    }

    public function hookCategoryUpdate($params)
    {
        $this->_clearCache('*');
    }

    public function hookActionAdminGroupsControllerSaveAfter($params)
    {
        $this->_clearCache('*');
    }

    public function _clearCache($template, $cache_id = null, $compile_id = null)
    {
        parent::_clearCache($this->templateFile);
    }

    public function getContent()
    {
        $output = '';
        $errors = array();

        if (Tools::isSubmit('submitHomeFeatured')) {
            
            // when saving, all files will be checked to be sure that name was converted.
            // THIS IS ONLY DURING DEVELOP ... REMOVE IN PRODUCTION
            //$this->installModuleFiles();
            
            
            $nbr = Tools::getValue($this->HOME_FEATURED_NBR);
            if (!Validate::isInt($nbr) || $nbr <= 0) {
                $errors[] = $this->l('The number of products is invalid. Please enter a positive number.');
            }

            $cat = Tools::getValue($this->HOME_FEATURED_CAT);
            if (!Validate::isInt($cat) || $cat <= 0) {
                $errors[] = $this->l('The category ID is invalid. Please choose an existing category ID.');
            }
            
            $brand = Tools::getValue($this->HOME_FEATURED_BRAND);
            if (!Validate::isInt($brand)) {
                $errors[] = $this->l('The brand ID is invalid. Please choose an existing brand ID.');
            }
            
            

            $rand = Tools::getValue($this->HOME_FEATURED_RANDOMIZE);
            if (!Validate::isBool($rand)) {
                $errors[] = $this->l('Invalid value for the "randomize" flag.');
            }
            
            
                
                
            $slide = Tools::getValue($this->HOME_FEATURED_SLIDE);
            if (!Validate::isBool($slide)) {
                $errors[] = $this->l('Invalid value for the "slide" flag.');
            }
            
            $best_sales = Tools::getValue($this->HOME_FEATURED_BEST_SALES);
            if (!Validate::isBool($slide)) {
                $errors[] = $this->l('Invalid value for the "best_sales" flag.');
            }
            
            
            
                
            $footer_link = Tools::getValue($this->HOME_FEATURED_FOOTER_LINK);
            if (!Validate::isBool($footer_link)) {
                $errors[] = $this->l('Invalid value for the "footer_link" flag.');
            }
            
            
            
            $enable_tag = Tools::getValue($this->HOME_FEATURED_ENABLE_TAG);
            if (!Validate::isBool($enable_tag)) {
                $errors[] = $this->l('Invalid value for the "enable tag" flag.');
            }
            
            $enable_product_category = Tools::getValue($this->HOME_FEATURED_PRODUCT_CATEGORY);
            if (!Validate::isBool($enable_product_category)) {
                $errors[] = $this->l('Invalid value for the "product category" flag.');
            }
            
            
            $enable_product_custom_css = Tools::getValue($this->PRODUCT_CUSTOM_CSS_ENABLE);
            if (!Validate::isBool($enable_product_custom_css)) {
                $errors[] = $this->l('Invalid value for the "enable product miniature custom css" flag.');
            }
            
            if (isset($errors) && count($errors)) {
                $output = $this->displayError(implode('<br />', $errors));
            } else {
                
                $languages = Language::getLanguages(false);
                $values = array();
                
                $values[$this->HOME_FEATURED_NBR]                 = (int) $nbr;
                $values[$this->HOME_FEATURED_CAT]                 = (int) $cat;
                $values[$this->HOME_FEATURED_BRAND]             = (int) $brand;
                
                $values[$this->HOME_FEATURED_RANDOMIZE]         = (bool) $rand;
                $values[$this->HOME_FEATURED_SLIDE]             = (bool) $slide;
                $values[$this->HOME_FEATURED_BEST_SALES]         = (bool) $best_sales;
                
                $values[$this->HOME_FEATURED_FOOTER_LINK]         = (bool) $footer_link;
                $values[$this->HOME_FEATURED_ENABLE_TAG]         = (bool) $enable_tag;
                $values[$this->HOME_FEATURED_PRODUCT_CATEGORY]     = (bool) $enable_product_category;
                $values[$this->PRODUCT_CUSTOM_CSS_ENABLE]         = (bool) $enable_product_custom_css;
                
                $values[$this->HOME_FEATURED_CLASS]             =  Tools::getValue($this->HOME_FEATURED_CLASS);
                
                foreach ($languages as $lang) {
                    $values[$this->HOME_FEATURED_DESC][$lang['id_lang']] = Tools::getValue($this->HOME_FEATURED_DESC . '_'.$lang['id_lang']);
                    $values[$this->HOME_FEATURED_SEARCH_STRING][$lang['id_lang']] = Tools::getValue($this->HOME_FEATURED_SEARCH_STRING . '_'.$lang['id_lang']);
                    
                }
                /*
                Configuration::updateValue($this->HOME_FEATURED_NBR, (int) $nbr);
                Configuration::updateValue($this->HOME_FEATURED_CAT, (int) $cat);
                Configuration::updateValue($this->HOME_FEATURED_RANDOMIZE, (bool) $rand);
                */
                
                
                Configuration::updateValue($this->HOME_FEATURED_NBR,                 (int) $values[$this->HOME_FEATURED_NBR]);
                Configuration::updateValue($this->HOME_FEATURED_CAT,                 (int) $values[$this->HOME_FEATURED_CAT]);
                Configuration::updateValue($this->HOME_FEATURED_BRAND,                 (int) $values[$this->HOME_FEATURED_BRAND]);
                
                Configuration::updateValue($this->HOME_FEATURED_RANDOMIZE,             (bool) $values[$this->HOME_FEATURED_RANDOMIZE]);
                Configuration::updateValue($this->HOME_FEATURED_SLIDE,                 (bool) $values[$this->HOME_FEATURED_SLIDE]);
                Configuration::updateValue($this->HOME_FEATURED_BEST_SALES,         (bool) $values[$this->HOME_FEATURED_BEST_SALES]);
                
                Configuration::updateValue($this->HOME_FEATURED_FOOTER_LINK,         (bool) $values[$this->HOME_FEATURED_FOOTER_LINK]);
                Configuration::updateValue($this->HOME_FEATURED_ENABLE_TAG,         (bool) $values[$this->HOME_FEATURED_ENABLE_TAG]);
                Configuration::updateValue($this->HOME_FEATURED_PRODUCT_CATEGORY,     (bool) $values[$this->HOME_FEATURED_PRODUCT_CATEGORY]);
                Configuration::updateValue($this->HOME_FEATURED_DESC,                 $values[$this->HOME_FEATURED_DESC]);
                Configuration::updateValue($this->HOME_FEATURED_SEARCH_STRING,         $values[$this->HOME_FEATURED_SEARCH_STRING]);
                Configuration::updateValue($this->HOME_FEATURED_CLASS,                 $values[$this->HOME_FEATURED_CLASS]);
                Configuration::updateValue($this->PRODUCT_CUSTOM_CSS_ENABLE,         (bool) $values[$this->PRODUCT_CUSTOM_CSS_ENABLE]);
                
                 // save custom css
                $this->saveCssFile(Tools::getValue($this->PRODUCT_CUSTOM_CSS_CONTENT));
                $this->_clearCache('*');
                $output = $this->displayConfirmation($this->trans('The settings have been updated.', array(), 'Admin.Notifications.Success'));
            }
        }
        
        return $output.$this->renderForm();
    }

    public function renderForm()
    {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Settings', array(), 'Admin.Global'),
                    'icon' => 'icon-cogs',
                ),

                'description' => $this->description
                                . '<br><ol><li>' . $this->l('Configure a default product slider to add to your homepage.') . '</li>'
                                . '<li>' . $this->l('Use the TAG system to add product sliders into CMS pages.') . '</li>'
                                . '<li>' . $this->l('Download mxb_customblocks and configure how many sliders you wish into your homepage.') 
                                . ' <a href="https://www.mxbinformatica.it/prestashop-mxb-customblocks/" target="_new">' . $this->l('Click here for more informations.') . '</a></li></ol>'
                                . '<br><b>' . $this->l('Documentation') . '</b>: <a href="https://www.mxbinformatica.it/prestashop-mxb-featuredproducts/" target="_blank">' . $this->l('click here') . '</a></li></ol>',
                'input' => array(
                    array(
                        'type' => 'text',
                        'label' => $this->l('Number of products to be displayed'),
                        'name' => $this->HOME_FEATURED_NBR,
                        'class' => 'fixed-width-xs',
                        'desc' => $this->l('Set the number of products that you would like to display on homepage (default: 8).'),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Category from which to pick products to be displayed'),
                        'name' => $this->HOME_FEATURED_CAT,
                        'class' => 'fixed-width-xs',
                        'desc' => $this->l('Choose the category ID of the products that you would like to display on homepage (default: 2 for "Home").'),
                    ),
                    
                    array(
                        'type' => 'text',
                        'label' => $this->l('Filter products by manufacturer'),
                        'name' => $this->HOME_FEATURED_BRAND,
                        'class' => 'fixed-width-xs',
                        'desc' => $this->l('Choose the manufacturer ID of the products that you would like to display on homepage. 0 to disable.'),
                    ),
                    
                    
                    array(
                        'type' => 'text',
                        'lang' => true,
                        'label' => $this->l('Text filter'),
                        'name' => $this->HOME_FEATURED_SEARCH_STRING,
                        'desc' => $this->l('Please enter a text to filter producs or leave it empty.')
                            . '<br><b>' . $this->l('NOTE: if search text is enabled, category and best sales filters will not be considered.') . '</b>' 
                    ),
                    
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Show Best Sales products'),
                        'name' => $this->HOME_FEATURED_BEST_SALES,
                        'class' => 'fixed-width-xs',
                        'desc' => $this->l('Enable if you wish to list best sales products. This option will exclude category filter (above).'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->trans('Yes', array(), 'Admin.Global'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->trans('No', array(), 'Admin.Global'),
                            ),
                        ),
                    ),
                        
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Randomly display featured products'),
                        'name' => $this->HOME_FEATURED_RANDOMIZE,
                        'class' => 'fixed-width-xs',
                        'desc' => $this->l('Enable if you wish the products to be displayed randomly (default: no).'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->trans('Yes', array(), 'Admin.Global'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->trans('No', array(), 'Admin.Global'),
                            ),
                        ),
                    ),
                    
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Show one row sliding thumbnails'),
                        'name' => $this->HOME_FEATURED_SLIDE,
                        'class' => 'fixed-width-xs',
                        'desc' => $this->l('Enable if you wish the products to be displayed in one row, with left and right arrows to slide it.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->trans('Yes', array(), 'Admin.Global'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->trans('No', array(), 'Admin.Global'),
                            ),
                        ),
                    ),
                    
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Show footer link'),
                        'name' => $this->HOME_FEATURED_FOOTER_LINK,
                        'class' => 'fixed-width-xs',
                        'desc' => $this->l('Disable if you wish to hide the "all products" link below the thumbnails block.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->trans('Yes', array(), 'Admin.Global'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->trans('No', array(), 'Admin.Global'),
                            ),
                        ),
                    ),
                    
                    
                    
                    
                    
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Display products for current category'),
                        'name' => $this->HOME_FEATURED_PRODUCT_CATEGORY,
                        'class' => 'fixed-width-xs',
                        'desc' => $this->l('Enable if you wish to display products of the current product category. Used only in product page (default: no).'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->trans('Yes', array(), 'Admin.Global'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->trans('No', array(), 'Admin.Global'),
                            ),
                        ),
                    ),
                    
                    
                    
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Enable module insert TAG'),
                        'name' => $this->HOME_FEATURED_ENABLE_TAG,
                        'class' => 'fixed-width-xs',
                        'desc' => array(
                                $this->l('Enable if you wish the products to be displayed where you put the [TAG] string.'),
                                $this->l('Sample of tag to insert in a CMS page:') 
                                . '<br><code>[' . $this->name . ' products=4, category=2, brand=0, random=1, position=top, slide=1, footer-link=0, filter-text=cotton, best-sales=0, class=my-custom-class, title=Intresting products!]</code>',
                                '<ul><li><strong>[position]</strong>: ' . $this->l('please use position=top, bottom or here. NOTE: if position is set to "here" the code tag will be substituted by the product list exactly where it is.') . '</li>'
                                . '<li><strong>[title] / [filter-text]</strong>: ' . $this->l('set it empty to hide title and avoid text search; omit options to use the default settings.') 
                                . '</li></ul>',
                                '<b>' . $this->l('If you have "mxb_customblocks" module installed') . '</b>: <br>' 
                                . $this->l('tag string can be put directly into a block description to obtain various blocks showing different products as a horizontally scrollable banners.'),
                        ),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->trans('Yes', array(), 'Admin.Global'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->trans('No', array(), 'Admin.Global'),
                            ),
                        ),
                    ),
                    
                    
                    
                    array(
                        'type' => 'text',
                        'lang' => true,
                        'label' => $this->l('Title for the view'),
                        'name' => $this->HOME_FEATURED_DESC,
                        'desc' => $this->l('Please enter a title to show at the top of the list. Let the field empty to hide title.')
                    ),
                
                    array(
                        'type' => 'text',
                        'label' => $this->l('Add a custom class name to mxb_featuredproducts section'),
                        'name' => $this->HOME_FEATURED_CLASS,
                        'desc' => $this->l('You may call it into custom css as "section.your-class-name". The best is to use this class for "module insert TAG" mode, adding a different class for every TAG to customize every single layout.'),
                    ),
                    
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Enable custom css for product miniature'),
                        'name' => $this->PRODUCT_CUSTOM_CSS_ENABLE,
                        'class' => 'fixed-width-xs',
                        'desc' => $this->l('Enable if you want to customize product minietures with following css.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->trans('Yes', array(), 'Admin.Global'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->trans('No', array(), 'Admin.Global'),
                            ),
                        ),
                    ),
                        
                    array(
                        'type' => 'textarea',
                        'lang' => false,
                        'label' => $this->l('Custom CSS for product miniatures'),
                        'name' => $this->PRODUCT_CUSTOM_CSS_CONTENT, 
                        'desc' => $this->l('You can modify this precompiled css to change colors, dimensions and other css parameters of product miniatures, only for this module.')
                                            . '<br><br><b>' . $this->l('Download DEFAULT CSS file') 
                                            . ': <a href="' . $this->getModulePathUri() . 'views/css/product.css" target="_new">product.css</a></b>'
                                            . '<br><b>' . $this->l('Take a look at product miniature TPL file to better understand CSS. You can look to the original current theme "product.tpl".') 
                                            //. ': <a href="' . $this->path2url($this->product_miniature_tpl) . '" target="_new">product.tpl</a></b>'
                                            . ': <a href="' . $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&download_product_tpl=1" target="_new">product.tpl</a></b>'
                                            
                    ),    
                ),
                    
                        
                'submit' => array(
                    'title' => $this->trans('Save', array(), 'Admin.Actions'),
                ),
            ),
        );

        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $helper->id = (int) Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitHomeFeatured';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($fields_form));
    }

    public function getConfigFieldsValues($forSmarty = false, $params = null)
    {
        /*
        return array(
            $this->HOME_FEATURED_NBR => Tools::getValue($this->HOME_FEATURED_NBR, (int) Configuration::get($this->HOME_FEATURED_NBR)),
            $this->HOME_FEATURED_CAT => Tools::getValue($this->HOME_FEATURED_CAT, (int) Configuration::get($this->HOME_FEATURED_CAT)),
            $this->HOME_FEATURED_RANDOMIZE => Tools::getValue($this->HOME_FEATURED_RANDOMIZE, (bool) Configuration::get($this->HOME_FEATURED_RANDOMIZE)),
        );
        */
        $languages = Language::getLanguages(false);
        $fields = array();

        $fields[$forSmarty ? 'HOME_FEATURED_NBR' : $this->HOME_FEATURED_NBR] = $params !== null && isset($params[$this->HOME_FEATURED_NBR]) ? $params[$this->HOME_FEATURED_NBR] : Tools::getValue($this->HOME_FEATURED_NBR, (int) Configuration::get($this->HOME_FEATURED_NBR));
        $fields[$forSmarty ? 'HOME_FEATURED_CAT' : $this->HOME_FEATURED_CAT] = $params !== null && isset($params[$this->HOME_FEATURED_CAT]) ? $params[$this->HOME_FEATURED_CAT] : Tools::getValue($this->HOME_FEATURED_CAT, (int) Configuration::get($this->HOME_FEATURED_CAT));
        $fields[$forSmarty ? 'HOME_FEATURED_BRAND' : $this->HOME_FEATURED_BRAND] = $params !== null && isset($params[$this->HOME_FEATURED_BRAND]) ? $params[$this->HOME_FEATURED_BRAND] : Tools::getValue($this->HOME_FEATURED_BRAND, (int) Configuration::get($this->HOME_FEATURED_BRAND));
        
        $fields[$forSmarty ? 'HOME_FEATURED_RANDOMIZE' : $this->HOME_FEATURED_RANDOMIZE] = $params !== null && isset($params[$this->HOME_FEATURED_RANDOMIZE]) ? $params[$this->HOME_FEATURED_RANDOMIZE] : Tools::getValue($this->HOME_FEATURED_RANDOMIZE, (bool) Configuration::get($this->HOME_FEATURED_RANDOMIZE));
        $fields[$forSmarty ? 'HOME_FEATURED_SLIDE' : $this->HOME_FEATURED_SLIDE] = $params !== null && isset($params[$this->HOME_FEATURED_SLIDE]) ? $params[$this->HOME_FEATURED_SLIDE] : Tools::getValue($this->HOME_FEATURED_SLIDE, (bool) Configuration::get($this->HOME_FEATURED_SLIDE));
        $fields[$forSmarty ? 'HOME_FEATURED_BEST_SALES' : $this->HOME_FEATURED_BEST_SALES] = $params !== null && isset($params[$this->HOME_FEATURED_BEST_SALES]) ? $params[$this->HOME_FEATURED_BEST_SALES] : Tools::getValue($this->HOME_FEATURED_BEST_SALES, (bool) Configuration::get($this->HOME_FEATURED_BEST_SALES));
        
        
        $fields[$forSmarty ? 'HOME_FEATURED_FOOTER_LINK' : $this->HOME_FEATURED_FOOTER_LINK] = $params !== null && isset($params[$this->HOME_FEATURED_FOOTER_LINK]) ? $params[$this->HOME_FEATURED_FOOTER_LINK] : Tools::getValue($this->HOME_FEATURED_FOOTER_LINK, (bool) Configuration::get($this->HOME_FEATURED_FOOTER_LINK));
        
        $fields[$forSmarty ? 'HOME_FEATURED_ENABLE_TAG' : $this->HOME_FEATURED_ENABLE_TAG] = Tools::getValue($this->HOME_FEATURED_ENABLE_TAG, (bool) Configuration::get($this->HOME_FEATURED_ENABLE_TAG));
        $fields[$forSmarty ? 'HOME_FEATURED_PRODUCT_CATEGORY' : $this->HOME_FEATURED_PRODUCT_CATEGORY] = Tools::getValue($this->HOME_FEATURED_PRODUCT_CATEGORY, (bool) Configuration::get($this->HOME_FEATURED_PRODUCT_CATEGORY));
        
        $fields[$forSmarty ? 'PRODUCT_CUSTOM_CSS_ENABLE' : $this->PRODUCT_CUSTOM_CSS_ENABLE] = Tools::getValue($this->PRODUCT_CUSTOM_CSS_ENABLE, (bool) Configuration::get($this->PRODUCT_CUSTOM_CSS_ENABLE));
        
        $fields[$forSmarty ? 'HOME_FEATURED_CLASS' : $this->HOME_FEATURED_CLASS] = $params !== null && isset($params[$this->HOME_FEATURED_CLASS]) ? $params[$this->HOME_FEATURED_CLASS] : Tools::getValue($this->HOME_FEATURED_CLASS, (string)Configuration::get($this->HOME_FEATURED_CLASS));
        
        foreach ($languages as $lang) {
            //$fields[$this->banner_img][$lang['id_lang']] = Tools::getValue($this->banner_img . '_'.$lang['id_lang'], Configuration::get($this->banner_img, $lang['id_lang']));
            $fields[$forSmarty ? 'HOME_FEATURED_DESC' : $this->HOME_FEATURED_DESC][$lang['id_lang']] = Tools::getValue($this->HOME_FEATURED_DESC . '_'.$lang['id_lang'], Configuration::get($this->HOME_FEATURED_DESC, $lang['id_lang']));
            $fields[$forSmarty ? 'HOME_FEATURED_SEARCH_STRING' : $this->HOME_FEATURED_SEARCH_STRING][$lang['id_lang']] = Tools::getValue($this->HOME_FEATURED_SEARCH_STRING . '_'.$lang['id_lang'], Configuration::get($this->HOME_FEATURED_SEARCH_STRING, $lang['id_lang']));
        }
        
        // get custom css
        $custom_css = $this->readCssFile();
        $fields[$forSmarty ? 'PRODUCT_CUSTOM_CSS_CONTENT' : $this->PRODUCT_CUSTOM_CSS_CONTENT] = Tools::getValue($this->PRODUCT_CUSTOM_CSS_CONTENT, $custom_css);

        return $fields;
        
    }
    
    
    public function installModuleFiles() {
        $this->installModuleFile('translations/it.php');
        $this->installModuleFile('views/templates/hook/mxb_featuredproducts.tpl');
        $this->installModuleFile('views/js/front.js');
        $this->installModuleFile('views/css/front.css');
        $this->installModuleFile('views/css/product.css');
        return true;
    }
    
   
   
    
    public function installModuleFile($module_file = 'translations/it.php', $overwrite = true)
    {
        $filedir = _PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR;
        $filename_src = $filedir . $module_file . '_'; // add underscore to open the source unmodifyed file
        $filename_tgt = $filedir . $module_file;
        $searchString = '_' . $this->rootName;
        
        if (!file_exists($filename_src))
            return false;
        if (file_exists($filename_tgt) && !$overwrite)
            return false;        
        
        $myfile = fopen($filename_src, "r");
        $filesize = filesize($filename_src);
        if ((int)$filesize === 0)
            return false;
        $content = fread($myfile, $filesize);
        fclose($myfile);
        
        // change default class with custom class and save file
        // text to replace not found ?
        //if (strpos($content, $searchString) === false)
        //    return true;
        // already replaced (replace text is already present) ?
        if (strpos($content, $this->name) === false)
            $content = str_replace($searchString, $this->name, $content); 
        
        // update the real file
        $myfile = fopen($filename_tgt, "w");
        fwrite($myfile, $content);
        fclose($myfile);
        
        return true;
    }
    
    protected function saveCssFile($content, $cssfile = 'product.css') {
        $filename = _PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR.$this->customCssFolder.DIRECTORY_SEPARATOR . $cssfile;
        $myfile = fopen($filename, "w");
        fwrite($myfile, $content);
        fclose($myfile);
    }
        
    protected function readCssFile($cssfile = 'product.css') {
        // check if file exists and create it if necessary
        $filename = _PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR.$this->customCssFolder.DIRECTORY_SEPARATOR . $cssfile;
        if (!file_exists($filename)) {
            $this->installModuleFile('views'.DIRECTORY_SEPARATOR.'css'.DIRECTORY_SEPARATOR.$cssfile, false);
            copy(_PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR.'css'.DIRECTORY_SEPARATOR . $cssfile, $filename);
        }
        $myfile = fopen($filename, "r");
        $content = fread($myfile, filesize($filename));
        fclose($myfile);
        return $content;
    }
   
    public function renderWidgetAjax($hookName = null, $params = null)
    {
        $variables = $this->getWidgetVariables($hookName, array(), $params);
        if (empty($variables)) {
            return false;
        }
        $variables['page']['page_name'] = 'cms';
        
        $this->context->smarty->assign($variables);
        Die( $this->context->smarty->fetch($this->templateFile));
    }

    public function renderWidget($hookName = null, array $configuration = [], $params = null)
    {
        
        // LogDebug::WriteLog('mxb_featuredproduct_one', 'mxb_featuredproduct_one::renderWidget', 'START');
        /* REMOVE CACHE
        if (!$this->isCached($this->templateFile, $this->getCacheId($this->name))) {
        */
            $variables = $this->getWidgetVariables($hookName, $configuration, $params);

            if (empty($variables)) {
                // LogDebug::WriteLog('mxb_featuredproduct_one', 'mxb_featuredproduct_one::renderWidget', 'variables EMPTY');
                return false;
            }


            $this->smarty->assign($variables);
        /*}
        return $this->fetch($this->templateFile, $this->getCacheId($this->name));
        */
        
        return $this->fetch($this->templateFile);
    }

    public function getWidgetVariables($hookName = null, array $configuration = [], $params = null)
    {
        // LogDebug::WriteLog('mxb_featuredproduct_one', 'mxb_featuredproduct_one::getWidgetVariables', 'START');
        if ($params === null) {
            $params = $this->getConfigFieldsValues();
        }
        $products = $this->getProducts($params);
        //$title = Configuration::get($this->HOME_FEATURED_DESC, $this->context->language->id);
        $title = $params[$this->HOME_FEATURED_DESC][$this->context->language->id];
        $show_footer_link = (int)$params[$this->HOME_FEATURED_FOOTER_LINK];

        /* no more used ...
        $product_miniature_tpl = file_exists(_PS_MODULE_DIR_.$this->name . '/views/templates/catalog/_partials/miniatures/product.tpl')
                                    ? _PS_MODULE_DIR_.$this->name . '/views/templates/catalog/_partials/miniatures/product.tpl'
                                    : _PS_THEME_DIR_.'../classic/templates/catalog/_partials/miniatures/product.tpl';
        */
        // load default product thumbnail
        //$product_miniature_tpl = _PS_THEME_DIR_.'templates/catalog/_partials/miniatures/product.tpl';
         
        if (!empty($products)) {
            return array(
                'products' => $products,
                'settings' => $this->getConfigFieldsValues(true, $params),
                'title' => $title,
                'show_footer_link' => $show_footer_link,
                'product_miniature_tpl' => $this->product_miniature_tpl,
                'allProductsLink' => Context::getContext()->link->getCategoryLink($params[$this->HOME_FEATURED_CAT]),
            );
        }
        return false;
    }



    protected function getProducts($params = null)
    {
        if ($params === null) {
            $params = $this->getConfigFieldsValues();
        }

        // if best_sales: do not consider category, random
        // if searchString: do not consider category and best_sales, random
        $nProducts = $params[$this->HOME_FEATURED_NBR];
        if ($nProducts < 0) {
            $nProducts = 12;
        }
        $search_string = $params[$this->HOME_FEATURED_SEARCH_STRING][$this->context->language->id];
        $best_sales = (bool)$params[$this->HOME_FEATURED_BEST_SALES];
        $id_manufacturer = (int)$params[$this->HOME_FEATURED_BRAND];
        $id_category = (int) $params[$this->HOME_FEATURED_CAT];
        
        $context = new ProductSearchContext($this->context);
        $query = new ProductSearchQuery();
        $query
            ->setResultsPerPage($nProducts)
            ->setPage(1)
        ;
        
        
        
        if (strlen($search_string)) {
            $query->setSearchString($search_string);
            $query->setSortOrder(new SortOrder('product', 'position', 'asc'));
            $searchProvider = new SearchProductSearchProvider(
                $this->context->getTranslator()
            );
            
            /* TEST DEBUG
            $result = Search::find(
                $this->context->language->id,
                $query->getSearchString(),
                $query->getPage(),
                $query->getResultsPerPage(),
                $query->getSortOrder()->toLegacyOrderBy(), //new SortOrder('product', 'position', 'asc'),
                $query->getSortOrder()->toLegacyOrderWay(),
                false, // ajax, what's the link?
                false, // $use_cookie, ignored anyway
                null
            );
            $products = $result['result'];
            $count = $result['total'];
            MxbToolsDebug::WriteLog($this->name, 'getProducts()', 'FIND str="'.$query->getSearchString().'" products='.print_r($products, true), true);
            */
        }
        elseif ($best_sales) {
            /*    $query->setSortOrder(new SortOrder('product', 'position', 'asc'));*/
            $searchProvider = new BestSalesProductSearchProvider(
                $this->context->getTranslator()
            );    
        }
        elseif ($id_manufacturer > 0) {
            /*    $query->setSortOrder(new SortOrder('product', 'position', 'asc'));*/
            
            $manufacturer = new Manufacturer($id_manufacturer, $this->context->language->id);
            
            $searchProvider = new ManufacturerProductSearchProvider(
                $this->context->getTranslator(),
                $manufacturer
            );    
        }
        else {
            
            // if disabled or if parameters comes from tag string (ajax.php), we must consider the configured category
            // tag string sample: [mxb_featuredproducts_one products=4, category=2, random=1, position=top, title=Intresting products!]
            if ( $params[$this->HOME_FEATURED_PRODUCT_CATEGORY] ) {
                
                // ricerca categoria del prodotto visualizzato
                if ( Tools::getIsset("controller") && Tools::getValue("controller") == "product" ) {
                    
                    if (Tools::getIsset("id_product")) {
                        
                        $id_product =  Tools::getValue("id_product");
                        $product = new Product((int)$id_product);
                         // LogDebug::WriteLog('mxb_featuredproduct_one', 'mxb_featuredproduct_one::getProducts', ' id_product='.$id_product);
                        if (Validate::isLoadedObject($product)) {
                            $cats = $product->getCategories();
                            $cnt = count($cats);
                            if ($cnt) {
                                
                                 // LogDebug::WriteLog('mxb_featuredproduct_one', 'mxb_featuredproduct_one::getProducts', ' cats='.print_r($cats, true));
                                // get the last element: it should be the product category, and not parent
                                $id_category = (int) $cats[$cnt-1];
                                // $id_category = $product->id_category_default;
                            }
                        }
                    }
                }
            }
            
       
            $category = new Category($id_category);
            $searchProvider = new CategoryProductSearchProvider(
                $this->context->getTranslator(),
                $category
            );
        
            if ($params[$this->HOME_FEATURED_RANDOMIZE]) {
                $query->setSortOrder(SortOrder::random());
            } else {
                $query->setSortOrder(new SortOrder('product', 'position', 'asc'));
            }
        } // category filter
        
        
        
        
        $result = $searchProvider->runQuery(
            $context,
            $query
        );

        $assembler = new ProductAssembler($this->context);

        $presenterFactory = new ProductPresenterFactory($this->context);
        $presentationSettings = $presenterFactory->getPresentationSettings();
        $presenter = new ProductListingPresenter(
            new ImageRetriever(
                $this->context->link
            ),
            $this->context->link,
            new PriceFormatter(),
            new ProductColorsRetriever(),
            $this->context->getTranslator()
        );

        $products_for_template = [];

        foreach ($result->getProducts() as $rawProduct) {
            $products_for_template[] = $presenter->present(
                $presentationSettings,
                $assembler->assembleProduct($rawProduct),
                $this->context->language
            );
        }

        return $products_for_template;
    }
    
    
    
    public function getModulePathUri() {
        // $uri =  Tools::getHttpHost(true).__PS_BASE_URI__ . DIRECTORY_SEPARATOR . 'modules' . DIRECTORY_SEPARATOR . $this->name . DIRECTORY_SEPARATOR;    // first method: OK
        $uri = $this->path2url($this->getModulePath());            // second method: OK
        if (substr($uri, -1) != DIRECTORY_SEPARATOR)
            $uri .= DIRECTORY_SEPARATOR;
        return $uri;
    }
    
    public function getModulePath() {
        return _PS_MODULE_DIR_ . $this->name . DIRECTORY_SEPARATOR;
    }
    
    public function path2url($file, $Protocol=null) {
        if ($Protocol === null)
            $Protocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . '://';        
        //MxbToolsDebug::WriteLog('mxb_tools', 'Mxb_ToolsUtils::path2url', 'file: '.$file.', HTTP_HOST:'.$_SERVER['HTTP_HOST'].', DOCUMENT_ROOT:'.$_SERVER['DOCUMENT_ROOT']);
        $DOCUMENT_ROOT = $_SERVER['DOCUMENT_ROOT'];
        if (substr($DOCUMENT_ROOT, -1) != DIRECTORY_SEPARATOR)
            $DOCUMENT_ROOT .= DIRECTORY_SEPARATOR;
        $HTTP_HOST = $_SERVER['HTTP_HOST'];
        if (substr($HTTP_HOST, -1) != DIRECTORY_SEPARATOR)
            $HTTP_HOST .= DIRECTORY_SEPARATOR;        
        return $Protocol.$HTTP_HOST.str_replace($DOCUMENT_ROOT, '', $file);
    }

}
